import logging
import sys

from functools import wraps

DEFAULT_LOGGER = None


def get_logger(new_logger: bool = False):
    global DEFAULT_LOGGER
    if new_logger or not DEFAULT_LOGGER:
        log_handler = logging.StreamHandler(sys.stdout)
        log_handler.setFormatter(logging.Formatter(
            "%(asctime)s [%(levelname)s] [%(filename)s:%(lineno)d] %(message)s"
        ))
        DEFAULT_LOGGER = logging.getLogger('defaultLogger')
        DEFAULT_LOGGER.addHandler(log_handler)
        DEFAULT_LOGGER.setLevel(logging.DEBUG)
    return DEFAULT_LOGGER


def log_it(func) -> None:
    @wraps(func)
    def __call__(*args, **kwargs):
        logger = DEFAULT_LOGGER
        if (
            len(args) > 0 and
            isinstance(args[0], object) and
            isinstance(getattr(args[0], '_logger', None), logging.Logger)
        ):
            logger = args[0]._logger
        if logger:
            logger.debug(
                f'-- {func.__module__}.{func.__qualname__}(args={args},kwargs={kwargs} --'
            )
        return func(*args, **kwargs)
    return __call__



import logging
import requests

from pagerduty_api import log_it
from pagerduty_api.abstract_client import AbstractClient
from typing import Any, Dict
from pagerduty_api.utils import is_service_name_compliant


class PagerDutyServicesClient(AbstractClient):

    def __init__(
        self,
        http_session: requests.Session = requests.Session(),
        endpoint: str = 'https://api.pagerduty.com',
        auth_token: str = None,
        logger: logging.Logger = None
    ):
        super().__init__(http_session, endpoint, auth_token, logger)
        self._default_url = f'{self._endpoint}/services'
        self._default_entity_name = 'services'

    @log_it
    def list_services(
        self,
        limit: int = 100,
        params: Dict[str, Any] = {},
        paginate: bool = True
    ):
        '''
        >>> import os
        >>> client = PagerDutyServicesClient(auth_token=os.environ.get('AUTH_TOKEN'))
        >>> services = client.list_services()
        Executing HTTP get on https://api.pagerduty.com/services
        >>> len(services) > 250
        True
        '''
        if not params:
            params = {
                'query': '',
                'sort_by': 'name',
                'limit': limit,
            }
        params['limit'] = params.get('limit', limit)
        return self.do_request(
            url=self._default_url,
            entity_name=self._default_entity_name,
            params=params,
            paginate=paginate
        )

    # def list_compliant_services(
    #     self,
    #     limit: int = 100,
    #     params: Dict[str, Any] = {},
    #     paginate: bool = True
    # ):
    #     '''
    #     >>> import os
    #     >>> client = PagerDutyServicesClient(auth_token=os.environ.get('AUTH_TOKEN'))
    #     >>> services = client.list_compliant_services()
    #     Executing HTTP get on https://api.pagerduty.com/services
    #     >>> len(services) > 200
    #     True
    #     '''
    #     services = self.list_services(limit, params, paginate)
    #     return [ 
    #         s for s in services
    #         if is_service_name_compliant(s['name'])
    #     ]

    def list_compliant_services(
        self,
        services: list=[Dict[str, Any]]
    ):
        '''
        >>> import os
        >>> client = PagerDutyServicesClient(auth_token=os.environ.get('AUTH_TOKEN'))
        >>> services = client.list_compliant_services()
        Executing HTTP get on https://api.pagerduty.com/services
        >>> len(services) > 200
        True
        '''
        self._logger.debug('Listing compliant services')
        return [ 
            s for s in services
            if is_service_name_compliant(s['name'])
        ]
    

        # return service ids
        # return service names


import logging
import requests

from datetime import datetime, timezone, timedelta
from pagerduty_api import log_it
from pagerduty_api.abstract_client import AbstractClient
from typing import Any, Dict


class PagerDutyChangeEventsClient(AbstractClient):

    def __init__(
        self,
        http_session: requests.Session = requests.Session(),
        endpoint: str = 'https://api.pagerduty.com',
        auth_token: str = None,
        logger: logging.Logger = None
    ):
        super().__init__(http_session, endpoint, auth_token, logger)
        self._default_url = f'{self._endpoint}/change_events'
        self._default_entity_name = 'change_events'
    
    @log_it
    def list_change_events(
        self,
        limit: int = 100,
        since: str = (datetime.now(timezone.utc).replace(hour=0, minute=0, second=0, microsecond=0) - timedelta(days=1)).isoformat(),
        params: Dict[str, Any] = {},
        paginate: bool = True
    ):
        '''
        Links to docs: https://developer.pagerduty.com/api-reference/c36fffe46ba3b-list-change-events
        '''
        if not params:
            params = {
                'query': '',
                'sort_by': 'name',
                'limit': limit,
                'since': since
            }
        params['limit'] = params.get('limit', limit)
        params['since'] = params.get('since', since)
        return self.do_request(
            url=self._default_url,
            entity_name=self._default_entity_name,
            params=params,
            paginate=paginate
        )

    


import requests
import logging

from pagerduty_api import log_it
from pagerduty_api.abstract_client import AbstractClient
from typing import Any, Dict, List

class PagerDutyAnalyticsClient(AbstractClient):

    def __init__(
        self,
        http_session: requests.Session = requests.Session(),
        endpoint: str = 'https://api.pagerduty.com',
        auth_token: str = None,
        logger: logging.Logger = None
    ):
        super().__init__(http_session, endpoint, auth_token, logger)
        self._default_url = f'{self._endpoint}/analytics'
        self._default_entity_name = 'data'

    @log_it
    def list_aggregated_services(
        self,
        filters: Dict[str, Any] = {},
        aggregate_unit: str = None
    ):
        '''
        >>> import os
        >>> client = PagerDutyAnalyticsClient(auth_token=os.environ.get('AUTH_TOKEN'))
        >>> analytics = client.list_aggregated_services(data=data)
        Executing HTTP get on https://api.pagerduty.com/metrics/incidents/services
        >>> len(analytics) >= 1
        True
        '''
        return self.do_analytics_request(
            url=f'{self._default_url}/metrics/incidents/services',
            entity_name=self._default_entity_name,
            filters=filters,
            method='post',
            aggregate_unit=aggregate_unit
        )

    @log_it
    def list_aggregated_teams(
        self,
        filters: Dict[str, Any] = {},
        aggregate_unit: str = None
    ):
        '''
        >>> import os
        >>> client = PagerDutyAnalyticsClient(auth_token=os.environ.get('AUTH_TOKEN'))
        >>> analytics = client.list_aggregated_teams(data=data)
        Executing HTTP get on https://api.pagerduty.com/metrics/incidents/teams
        >>> len(analytics) >= 1
        True
        '''
        return self.do_analytics_request(
            url=f'{self._default_url}/metrics/incidents/teams',
            entity_name=self._default_entity_name,
            filters=filters,
            method='post',
            aggregate_unit=aggregate_unit
        )

    @log_it
    def list_aggregated_incidents(
        self,
        filters: Dict[str, Any] = {}
    ):
        '''
        >>> import os
        >>> client = PagerDutyAnalyticsClient(auth_token=os.environ.get('AUTH_TOKEN'))
        >>> incidents = client.list_aggregated_incidents(data=data)
        Executing HTTP get on https://api.pagerduty.com/incidents
        >>> len(incidents) >= 1
        True
        '''
        
        return self.do_analytics_request(
            url=f'{self._default_url}/metrics/incidents/all',
            entity_name=self._default_entity_name,
            filters=filters,
            method='post'
        )
    
    @log_it
    def list_raw_incidents(
        self,
        filters: Dict[str, Any] = None,
        limit: int = None,
        paginate: bool = False
    ):
        '''
        >>> import os
        >>> client = PagerDutyAnalyticsClient(auth_token=os.environ.get('AUTH_TOKEN'))
        >>> incidents = client.list_raw_incidents(data=data)
        Executing HTTP get on https://api.pagerduty.com/analytics/raw/incidents
        >>> len(incidents) >= 1
        True
        '''
        
        return self.do_analytics_request(
            url=f'{self._default_url}/raw/incidents',
            entity_name=self._default_entity_name,
            filters=filters or {},
            method='post',
            limit=limit,
            paginate=paginate
        )

    
#["P6THBO4", "PAOR13M", "PCTB88X", "PYPJAA9", "PG2XNDN", "PQI2LK3", "PNHSHMO", "PEG1GVL", "PRX56PU", "PG65ZZE"]
# feed service ids in batches of 10
# get incident count from here
